#include <Relay.h>
#include <Sensor.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <RF24.h>
#include <Bounce2.h>
#include <DHT.h>  
#define RELAY_1  7  
#define NUMBER_OF_RELAYS 2 
#define RELAY_ON 0
#define RELAY_OFF 1
#define BUTTON_PIN1  3  // Arduino Digital I/O pin for button/reed switch
#define BUTTON_PIN2  4  // Arduino Digital I/O pin for button/reed switch
#define BUTTON_PIN3  5  // Arduino Digital I/O pin for button/reed switch
#define BUTTON_PIN4  6  // Arduino Digital I/O pin for button/reed switch
#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#define HUMIDITY_SENSOR_DIGITAL_PIN 8
//
Sensor gw;
Bounce debouncer1 = Bounce();
Bounce debouncer2 = Bounce();
Bounce debouncer3 = Bounce();
Bounce debouncer4 = Bounce();
int oldValue1 = -1;
int oldValue2 = -1;
int oldValue3 = -1;
int oldValue4 = -1;
DHT dht;
float lastTemp;
float lastHum;
boolean metric = true; 
void setup()  
{  
  gw.begin();
  //
  pinMode(BUTTON_PIN1,INPUT);
  digitalWrite(BUTTON_PIN1,HIGH);
  pinMode(BUTTON_PIN2,INPUT);
  digitalWrite(BUTTON_PIN2,HIGH);
  pinMode(BUTTON_PIN3,INPUT);
  digitalWrite(BUTTON_PIN3,HIGH);
  pinMode(BUTTON_PIN4,INPUT);
  digitalWrite(BUTTON_PIN4,HIGH);
  //
  dht.setup(HUMIDITY_SENSOR_DIGITAL_PIN);
  // 
  debouncer1.attach(BUTTON_PIN1);
  debouncer1.interval(5);
  debouncer2.attach(BUTTON_PIN2);
  debouncer2.interval(5);
  debouncer3.attach(BUTTON_PIN3);
  debouncer3.interval(5);
  debouncer4.attach(BUTTON_PIN4);
  debouncer4.interval(5);
  //
  gw.sendSensorPresentation(BUTTON_PIN1, S_DOOR);  
  gw.sendSensorPresentation(BUTTON_PIN2, S_DOOR);  
  gw.sendSensorPresentation(BUTTON_PIN3, S_DOOR);  
  gw.sendSensorPresentation(BUTTON_PIN4, S_DOOR);  
  gw.sendSketchInfo("4Door2Relay", "1.0");
  //
  gw.sendSensorPresentation(CHILD_ID_HUM, S_HUM);
  gw.sendSensorPresentation(CHILD_ID_TEMP, S_TEMP);
  metric = gw.isMetricSystem();
  // 
  for (int i=0; i<NUMBER_OF_RELAYS;i++) 
  {
    gw.sendSensorPresentation(RELAY_1+i, S_LIGHT);
  }
  for (int i=0; i<NUMBER_OF_RELAYS;i++) 
  {
    digitalWrite(RELAY_1+i, RELAY_OFF);
    pinMode(RELAY_1+i, OUTPUT);   
    gw.getStatus(RELAY_1+i, V_LIGHT);
    setRelayStatus(gw.getMessage()); 
  }
}
//
void loop() 
{
  debouncer1.update();
  // Get the update value
  int value = debouncer1.read();
  if (value != oldValue1) 
  {
    gw.sendVariable(BUTTON_PIN1, V_TRIPPED, value==HIGH ? "1" : "0");
    oldValue1 = value;
  }
  //
  debouncer2.update();
  value = debouncer2.read();
  if (value != oldValue2) 
  {
    gw.sendVariable(BUTTON_PIN2, V_TRIPPED, value==HIGH ? "1" : "0");
    oldValue2 = value;
  }
  //
  debouncer3.update();
  value = debouncer3.read();
  if (value != oldValue3) 
  {
    gw.sendVariable(BUTTON_PIN3, V_TRIPPED, value==HIGH ? "1" : "0"); 
    oldValue3 = value;
  } 
  //
  debouncer4.update();
  value = debouncer4.read();
  if (value != oldValue4) 
  {
    gw.sendVariable(BUTTON_PIN4, V_TRIPPED, value==HIGH ? "1" : "0");
    oldValue4 = value;
  } 
  delay(dht.getMinimumSamplingPeriod());
  float temperature = dht.getTemperature();
  if (isnan(temperature)) 
  {
    Serial.println("Failed reading temperature from DHT");
  } 
  else if (temperature != lastTemp) 
  {
    lastTemp = temperature;
    if (!metric) {
      temperature = dht.toFahrenheit(temperature);
    }
    gw.sendVariable(CHILD_ID_TEMP, V_TEMP, temperature, 1);
    Serial.print("T: ");
    Serial.println(temperature);
  }
  float humidity = dht.getHumidity();
  if (isnan(humidity)) 
  {
    Serial.println("Failed reading humidity from DHT");
  } 
  else if (humidity != lastHum) 
  {
    lastHum = humidity;
    gw.sendVariable(CHILD_ID_HUM, V_HUM, humidity, 1);
    Serial.print("H: ");
    Serial.println(humidity);
  }
  if (gw.messageAvailable()) 
  {
    message_s message = gw.getMessage(); 
    setRelayStatus(message);
  }
}
//
void setRelayStatus(message_s message) 
{
  if (message.header.messageType==M_SET_VARIABLE && message.header.type==V_LIGHT) 
  {
    int incomingRelayStatus = atoi(message.data);
    // Change relay state
    digitalWrite(message.header.childId, incomingRelayStatus==1?RELAY_ON:RELAY_OFF);
    // Write some debug info
    Serial.print("Incoming change for relay on pin:");
    Serial.print(message.header.childId);
    Serial.print(", New status: ");
    Serial.println(incomingRelayStatus);
  }
}

